COMPONENTS = {
	// a comment
	
	"EvenPlusOdd" : [
		"+Even",
		"+Odd"
	],
	
	"Even" : [
		"File2",
		"File4"
    ],
    
    
    "Odd" : [
    	"File1",
    	"File3"
    ],
    
    "Part21" : [
    	"+Part22",
    	"Part2.File1",
    	"=jsan/lib/File4.js"
    ],
    
    "Part22" : [
    	"+Part23",
    	"Part2.File2"
    ],
    
    "Part23" : [
    	"jsan:File1",
    	"Part2.File3",
    	"jsan:File2"
    ]
}